#version 3.7;

#ifndef(TestMacros)
	#declare TestMacros = false;
#end


/*
#declare Iterations = 24;
#declare Mod = function (Raw) {mod (Raw, 4)}

#for (n, 1, Iterations, 1)
	#declare e = n;
	#declare E = Mod (e);
	#debug concat (str (n, 2, 0), " | e = ", str (e, 2, 3), " | Mod = ", str (E, 2, 3), "\n")
#end
*/


//###############################################

#macro DisplayTextArray (_Array)
	// display contents of scalar array to _P decimal places
	#debug concat ("[Text]Array = {",
	#for (_X, 0, dimension_size (_Array, 1)-2)
		_Array [_X], ", ",
	#end
	_Array [_X], "}\n\n")
#end

//###############################################

#macro DisplayScalarArray (_Array, _P)
	// display contents of scalar array to _P decimal places
	#debug concat ("[Scalar]Array = {",
	#for (_X, 0, dimension_size (_Array, 1)-2)
		str(_Array [_X], 0, _P), ", ",
	#end
	str(_Array [_X], 0, _P), "}\n\n")
#end

//###############################################

#macro Scalar (_Value, _P, _N)
	#local _Result = str(_Value, 0, _P);
	#if(_N) #local _Result = concat (_Result, " \n") #end
	_Result
#end

//###############################################

#macro Vector (_Value, _P, _N)
	#local _Result = concat ("<", vstr(3, _Value, ", ", 0, _P), ">");
	#if(_N) #local _Result = concat (_Result, " \n") #end
	_Result
#end

//###############################################

#macro ConvertScalarArrayToText (_Array, _P)
	// display contents of scalar array to _P decimal places
	#local _Result = array [dimension_size (_Array, 1)];
	#for (_X, 0, dimension_size (_Array, 1)-1)
		#local _Result [_X] = str(_Array [_X], 0, _P);
	#end
	_Result
#end

//###############################################

#macro DisplayVectorArray (_Array, _P)
	// display contents of vector array to _P decimal places
	#debug concat ("[Vector]Array = {",
	#for (_X, 0, dimension_size (_Array, 1)-2)
		"<", vstr(3, _Array [_X], ", ", 0, _P), ">, ",
	#end
	"<", vstr(3, _Array [_X], ", ", 0, _P), ">}\n\n")
#end

//###############################################

#macro ConvertVectorArrayToText (_Array, _P)
	// display contents of scalar array to _P decimal places
	#local _Result = array [dimension_size (_Array, 1)];
	#for (_X, 0, dimension_size (_Array, 1)-1)
		#local _Result [_X] = concat ("<", vstr(3, _Array [_X], ", ", 0, _P), ">");
	#end
	_Result
#end

//###############################################


#macro Choose (_N, _Array)
	// returns an element from an array
	// uses mod () to cycle through array no matter the size of _N or _Array
	
	// *** Add optional dimension to access other than 1

	#local __N = abs(_N);
	#local _Size = dimension_size (_Array,1);
	#local _Item = mod (_N, _Size);
	#local _Result = _Array [_Item];
	
_Result
#end

#if (TestMacros)
	// test macro Choose()
	#debug "Testing macro Choose (_N, _Array) \n"
	#declare List = array [6] {"A", "B", "C", "D", "E", "F"};
	#declare C = Choose (34, List);
	#debug concat (C, "\n\n")
#end

//###############################################

#macro Populate (_Item, _Qty)
	// creates a new array of N copies of an element
	#local _Array = array [_Qty];
		#for (_X, 0, _Qty-1)
			#local _Array [_X] = _Item;
		#end
	_Array
#end

#if (TestMacros)
	// test macro Populate()
	#debug "Testing macro Populate (I, Q) \n"
	#declare Filled = Populate (5, 10);
	
	DisplayScalarArray (Filled, 0)
#end

//###############################################

#macro IncrementArray (_Start, _End, _Step)
	// creates a new array with values from N to M, step S
	#local _Qty = (_End - _Start) / _Step;
	#local _Array = array [_Qty+1];
		#for (_X, 0, _Qty)
			#local _Item = _Start + (_Step * _X);
			#local _Array [_X] = _Item;
		#end
	_Array
#end

#if (TestMacros)
	// test macro IncrementArray()
	#debug "Testing macro IncrementArray (5, 10, 0.5) \n"
	#declare Incremental = IncrementArray (5, 10, 0.5);
	
	DisplayScalarArray (Incremental, 1)
	
	#declare IncrementalText = ConvertScalarArrayToText (Incremental, 1);
	#debug "Testing macro ConvertScalarArrayToText \n"
	DisplayTextArray (IncrementalText)

#end
//###############################################

#macro Multiply_Array (_Array, _Multiplier)
	// Multiplies all the elements of an array by a multiplier
	// Multiplier may be either a scalar or a vector
	// Returns a new array of the products
	#local _Temp = array [dimension_size(_Array,1)];
		#for (_X, 0, dimension_size(_Array,1)-1)
			#local _Temp [_X] = _Array[_X] * _Multiplier;
		#end
	_Temp
#end

#if (TestMacros)
	// test macro Multiply_Array
	#declare MultiplyMe = array [8] {1,2,3,4,5,6,7,8};
	#declare Multiplied = Multiply_Array (MultiplyMe, <1, 1, 1>);
	
	#debug "Testing macro Multiply_Array (Array, Multiplier) \n"
	DisplayVectorArray (Multiplied, 0)
#end

//###############################################

#macro Multiply_Arrays (_Array1, _Array2)
	// Multiplies two arrays of equal size together
	// Multiplicands may be either scalars or vectors
	// Returns a new array of the products
	#local _D1 = dimension_size(_Array1,1);
	#local _D2 = dimension_size(_Array2,1);
	
	#local _Temp = array [_D1];
		#for (_X, 0, _D1-1)
			#local _Temp [_X] = _Array1[_X] * _Array2[_X];
		#end
	_Temp
#end

#if (TestMacros)
	// test macro Multiply_Arrays
	#declare MultiplyMe1 = array [5] {1,3,5,7,9};
	#declare MultiplyMe2 = array [5] {0,2,4,6,8};
	#declare Multiplied = Multiply_Arrays (MultiplyMe1, MultiplyMe2);
	
	#debug "Testing macro Multiply_Array (Array, Multiplier) \n"
	DisplayScalarArray (Multiplied, 0)
	
	#declare MultipliedText = ConvertVectorArrayToText (Multiplied, 0);
	#debug "Testing macro ConvertVectorArrayToText \n"
	DisplayTextArray (MultipliedText)
	#debug "(Look, kids - vector promotion! :)  ) \n\n\n"
#end

//###############################################

#macro Factorial (N)
	#local Result = 1;
	#local Ind = 2;
	#while (Ind <= N)
		#local Result = Result * Ind;
		#local Ind = Ind + 1;
	#end
	Result
#end

//###############################################

#macro NChooseM (_N, _M)
// nCm = n! / (k! * (n-m)!)
	#local _Result = Factorial (_N) / (Factorial (_M) * Factorial (_N - _M));
	_Result
#end

#if (TestMacros)
	#debug "Testing macro NChooseM (N, M) [uses macro Factorial (N) ] \n"
	#declare F42 = NChooseM (4, 2);
	#debug concat ("4 choose 2 = ", str(F42, 1, 0), "\n\n")
#end
//###############################################

#declare Cycle = function (_N, _Size) {mod (select (_N, (_Size + _N), 0, _N), _Size)}


#if (TestMacros)
	#debug "Testing function (N, Size) \n"
	#for (_X, -5, 20)
		#debug concat ("Item (",str(_X, 1, 0),"/", str(4, 1, 0),") = " str(Cycle(_X, 4), 1, 0), "\n")
	#end
#end

#ifndef (SDL)
	#error "end"
#end


